# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import (
    custom_field_add_key_params,
    custom_field_list_keys_params,
    custom_field_remove_key_params,
    custom_field_set_values_params,
    custom_field_delete_values_params,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.custom_field_list_keys_response import CustomFieldListKeysResponse

__all__ = ["CustomFieldsResource", "AsyncCustomFieldsResource"]


class CustomFieldsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomFieldsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return CustomFieldsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomFieldsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return CustomFieldsResourceWithStreamingResponse(self)

    def add_key(
        self,
        *,
        enforce_uniqueness: bool,
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Add a key to the allow list for a given entity.

        There is a 100 character limit
        on custom field keys.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/v1/customFields/addKey",
            body=maybe_transform(
                {
                    "enforce_uniqueness": enforce_uniqueness,
                    "entity": entity,
                    "key": key,
                },
                custom_field_add_key_params.CustomFieldAddKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def delete_values(
        self,
        *,
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        entity_id: str,
        keys: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Deletes one or more custom fields on an instance of a Metronome entity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/v1/customFields/deleteValues",
            body=maybe_transform(
                {
                    "entity": entity,
                    "entity_id": entity_id,
                    "keys": keys,
                },
                custom_field_delete_values_params.CustomFieldDeleteValuesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list_keys(
        self,
        *,
        next_page: str | NotGiven = NOT_GIVEN,
        entities: List[
            Literal[
                "alert",
                "billable_metric",
                "charge",
                "commit",
                "contract_credit",
                "contract_product",
                "contract",
                "credit_grant",
                "customer_plan",
                "customer",
                "discount",
                "invoice",
                "plan",
                "professional_service",
                "product",
                "rate_card",
                "scheduled_charge",
                "subscription",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomFieldListKeysResponse:
        """
        List all active custom field keys, optionally filtered by entity type.

        Args:
          next_page: Cursor that indicates where the next page of results should start.

          entities: Optional list of entity types to return keys for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/customFields/listKeys",
            body=maybe_transform({"entities": entities}, custom_field_list_keys_params.CustomFieldListKeysParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"next_page": next_page}, custom_field_list_keys_params.CustomFieldListKeysParams
                ),
            ),
            cast_to=CustomFieldListKeysResponse,
        )

    def remove_key(
        self,
        *,
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Remove a key from the allow list for a given entity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/v1/customFields/removeKey",
            body=maybe_transform(
                {
                    "entity": entity,
                    "key": key,
                },
                custom_field_remove_key_params.CustomFieldRemoveKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def set_values(
        self,
        *,
        custom_fields: Dict[str, str],
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        entity_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Sets one or more custom fields on an instance of a Metronome entity.

        If a
        key/value pair passed in this request matches one already set on the entity, its
        value will be overwritten. Any key/value pairs that exist on the entity that do
        not match those passed in this request will remain untouched. This endpoint is
        transactional and will update all key/value pairs or no key/value pairs. Partial
        updates are not supported. There is a 200 character limit on custom field
        values.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/v1/customFields/setValues",
            body=maybe_transform(
                {
                    "custom_fields": custom_fields,
                    "entity": entity,
                    "entity_id": entity_id,
                },
                custom_field_set_values_params.CustomFieldSetValuesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncCustomFieldsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomFieldsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomFieldsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomFieldsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncCustomFieldsResourceWithStreamingResponse(self)

    async def add_key(
        self,
        *,
        enforce_uniqueness: bool,
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Add a key to the allow list for a given entity.

        There is a 100 character limit
        on custom field keys.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/v1/customFields/addKey",
            body=await async_maybe_transform(
                {
                    "enforce_uniqueness": enforce_uniqueness,
                    "entity": entity,
                    "key": key,
                },
                custom_field_add_key_params.CustomFieldAddKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def delete_values(
        self,
        *,
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        entity_id: str,
        keys: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Deletes one or more custom fields on an instance of a Metronome entity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/v1/customFields/deleteValues",
            body=await async_maybe_transform(
                {
                    "entity": entity,
                    "entity_id": entity_id,
                    "keys": keys,
                },
                custom_field_delete_values_params.CustomFieldDeleteValuesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def list_keys(
        self,
        *,
        next_page: str | NotGiven = NOT_GIVEN,
        entities: List[
            Literal[
                "alert",
                "billable_metric",
                "charge",
                "commit",
                "contract_credit",
                "contract_product",
                "contract",
                "credit_grant",
                "customer_plan",
                "customer",
                "discount",
                "invoice",
                "plan",
                "professional_service",
                "product",
                "rate_card",
                "scheduled_charge",
                "subscription",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomFieldListKeysResponse:
        """
        List all active custom field keys, optionally filtered by entity type.

        Args:
          next_page: Cursor that indicates where the next page of results should start.

          entities: Optional list of entity types to return keys for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/customFields/listKeys",
            body=await async_maybe_transform(
                {"entities": entities}, custom_field_list_keys_params.CustomFieldListKeysParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"next_page": next_page}, custom_field_list_keys_params.CustomFieldListKeysParams
                ),
            ),
            cast_to=CustomFieldListKeysResponse,
        )

    async def remove_key(
        self,
        *,
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Remove a key from the allow list for a given entity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/v1/customFields/removeKey",
            body=await async_maybe_transform(
                {
                    "entity": entity,
                    "key": key,
                },
                custom_field_remove_key_params.CustomFieldRemoveKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def set_values(
        self,
        *,
        custom_fields: Dict[str, str],
        entity: Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ],
        entity_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Sets one or more custom fields on an instance of a Metronome entity.

        If a
        key/value pair passed in this request matches one already set on the entity, its
        value will be overwritten. Any key/value pairs that exist on the entity that do
        not match those passed in this request will remain untouched. This endpoint is
        transactional and will update all key/value pairs or no key/value pairs. Partial
        updates are not supported. There is a 200 character limit on custom field
        values.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/v1/customFields/setValues",
            body=await async_maybe_transform(
                {
                    "custom_fields": custom_fields,
                    "entity": entity,
                    "entity_id": entity_id,
                },
                custom_field_set_values_params.CustomFieldSetValuesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class CustomFieldsResourceWithRawResponse:
    def __init__(self, custom_fields: CustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.add_key = to_raw_response_wrapper(
            custom_fields.add_key,
        )
        self.delete_values = to_raw_response_wrapper(
            custom_fields.delete_values,
        )
        self.list_keys = to_raw_response_wrapper(
            custom_fields.list_keys,
        )
        self.remove_key = to_raw_response_wrapper(
            custom_fields.remove_key,
        )
        self.set_values = to_raw_response_wrapper(
            custom_fields.set_values,
        )


class AsyncCustomFieldsResourceWithRawResponse:
    def __init__(self, custom_fields: AsyncCustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.add_key = async_to_raw_response_wrapper(
            custom_fields.add_key,
        )
        self.delete_values = async_to_raw_response_wrapper(
            custom_fields.delete_values,
        )
        self.list_keys = async_to_raw_response_wrapper(
            custom_fields.list_keys,
        )
        self.remove_key = async_to_raw_response_wrapper(
            custom_fields.remove_key,
        )
        self.set_values = async_to_raw_response_wrapper(
            custom_fields.set_values,
        )


class CustomFieldsResourceWithStreamingResponse:
    def __init__(self, custom_fields: CustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.add_key = to_streamed_response_wrapper(
            custom_fields.add_key,
        )
        self.delete_values = to_streamed_response_wrapper(
            custom_fields.delete_values,
        )
        self.list_keys = to_streamed_response_wrapper(
            custom_fields.list_keys,
        )
        self.remove_key = to_streamed_response_wrapper(
            custom_fields.remove_key,
        )
        self.set_values = to_streamed_response_wrapper(
            custom_fields.set_values,
        )


class AsyncCustomFieldsResourceWithStreamingResponse:
    def __init__(self, custom_fields: AsyncCustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.add_key = async_to_streamed_response_wrapper(
            custom_fields.add_key,
        )
        self.delete_values = async_to_streamed_response_wrapper(
            custom_fields.delete_values,
        )
        self.list_keys = async_to_streamed_response_wrapper(
            custom_fields.list_keys,
        )
        self.remove_key = async_to_streamed_response_wrapper(
            custom_fields.remove_key,
        )
        self.set_values = async_to_streamed_response_wrapper(
            custom_fields.set_values,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["UsageIngestParams", "Usage"]


class UsageIngestParams(TypedDict, total=False):
    usage: Iterable[Usage]


class Usage(TypedDict, total=False):
    customer_id: Required[str]

    event_type: Required[str]

    timestamp: Required[str]
    """RFC 3339 formatted"""

    transaction_id: Required[str]

    properties: Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorPage, AsyncCursorPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.v1.customers import (
    plan_add_params,
    plan_end_params,
    plan_list_params,
    plan_list_price_adjustments_params,
)
from ....types.v1.customers.plan_add_response import PlanAddResponse
from ....types.v1.customers.plan_end_response import PlanEndResponse
from ....types.v1.customers.plan_list_response import PlanListResponse
from ....types.v1.customers.plan_list_price_adjustments_response import PlanListPriceAdjustmentsResponse

__all__ = ["PlansResource", "AsyncPlansResource"]


class PlansResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return PlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return PlansResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        customer_id: str,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[PlanListResponse]:
        """
        List the given customer's plans in reverse-chronological order.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/plans",
            page=SyncCursorPage[PlanListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_params.PlanListParams,
                ),
            ),
            model=PlanListResponse,
        )

    def add(
        self,
        *,
        customer_id: str,
        plan_id: str,
        starting_on: Union[str, datetime],
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        net_payment_terms_days: float | NotGiven = NOT_GIVEN,
        overage_rate_adjustments: Iterable[plan_add_params.OverageRateAdjustment] | NotGiven = NOT_GIVEN,
        price_adjustments: Iterable[plan_add_params.PriceAdjustment] | NotGiven = NOT_GIVEN,
        trial_spec: plan_add_params.TrialSpec | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanAddResponse:
        """Associate an existing customer with a plan for a specified date range.

        See the
        [price adjustments documentation](https://plans-docs.metronome.com/pricing/managing-plans/#price-adjustments)
        for details on the price adjustments.

        Args:
          starting_on: RFC 3339 timestamp for when the plan becomes active for this customer. Must be
              at 0:00 UTC (midnight).

          ending_before: RFC 3339 timestamp for when the plan ends (exclusive) for this customer. Must be
              at 0:00 UTC (midnight).

          net_payment_terms_days: Number of days after issuance of invoice after which the invoice is due (e.g.
              Net 30).

          overage_rate_adjustments: An optional list of overage rates that override the rates of the original plan
              configuration. These new rates will apply to all pricing ramps.

          price_adjustments: A list of price adjustments can be applied on top of the pricing in the plans.
              See the
              [price adjustments documentation](https://plans-docs.metronome.com/pricing/managing-plans/#price-adjustments)
              for details.

          trial_spec: A custom trial can be set for the customer's plan. See the
              [trial configuration documentation](https://docs.metronome.com/provisioning/configure-trials/)
              for details.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._post(
            f"/v1/customers/{customer_id}/plans/add",
            body=maybe_transform(
                {
                    "plan_id": plan_id,
                    "starting_on": starting_on,
                    "ending_before": ending_before,
                    "net_payment_terms_days": net_payment_terms_days,
                    "overage_rate_adjustments": overage_rate_adjustments,
                    "price_adjustments": price_adjustments,
                    "trial_spec": trial_spec,
                },
                plan_add_params.PlanAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanAddResponse,
        )

    def end(
        self,
        *,
        customer_id: str,
        customer_plan_id: str,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        void_invoices: bool | NotGiven = NOT_GIVEN,
        void_stripe_invoices: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanEndResponse:
        """
        Change the end date of a customer's plan.

        Args:
          ending_before: RFC 3339 timestamp for when the plan ends (exclusive) for this customer. Must be
              at 0:00 UTC (midnight). If not provided, the plan end date will be cleared.

          void_invoices: If true, plan end date can be before the last finalized invoice date. Any
              invoices generated after the plan end date will be voided.

          void_stripe_invoices: Only applicable when void_invoices is set to true. If true, for every invoice
              that is voided we will also attempt to void/delete the stripe invoice (if any).
              Stripe invoices will be voided if finalized or deleted if still in draft state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not customer_plan_id:
            raise ValueError(f"Expected a non-empty value for `customer_plan_id` but received {customer_plan_id!r}")
        return self._post(
            f"/v1/customers/{customer_id}/plans/{customer_plan_id}/end",
            body=maybe_transform(
                {
                    "ending_before": ending_before,
                    "void_invoices": void_invoices,
                    "void_stripe_invoices": void_stripe_invoices,
                },
                plan_end_params.PlanEndParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanEndResponse,
        )

    def list_price_adjustments(
        self,
        *,
        customer_id: str,
        customer_plan_id: str,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[PlanListPriceAdjustmentsResponse]:
        """Lists a customer plans adjustments.

        See the
        [price adjustments documentation](https://plans-docs.metronome.com/pricing/managing-plans/#price-adjustments)
        for details.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not customer_plan_id:
            raise ValueError(f"Expected a non-empty value for `customer_plan_id` but received {customer_plan_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/plans/{customer_plan_id}/priceAdjustments",
            page=SyncCursorPage[PlanListPriceAdjustmentsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_price_adjustments_params.PlanListPriceAdjustmentsParams,
                ),
            ),
            model=PlanListPriceAdjustmentsResponse,
        )


class AsyncPlansResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncPlansResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        customer_id: str,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PlanListResponse, AsyncCursorPage[PlanListResponse]]:
        """
        List the given customer's plans in reverse-chronological order.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/plans",
            page=AsyncCursorPage[PlanListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_params.PlanListParams,
                ),
            ),
            model=PlanListResponse,
        )

    async def add(
        self,
        *,
        customer_id: str,
        plan_id: str,
        starting_on: Union[str, datetime],
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        net_payment_terms_days: float | NotGiven = NOT_GIVEN,
        overage_rate_adjustments: Iterable[plan_add_params.OverageRateAdjustment] | NotGiven = NOT_GIVEN,
        price_adjustments: Iterable[plan_add_params.PriceAdjustment] | NotGiven = NOT_GIVEN,
        trial_spec: plan_add_params.TrialSpec | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanAddResponse:
        """Associate an existing customer with a plan for a specified date range.

        See the
        [price adjustments documentation](https://plans-docs.metronome.com/pricing/managing-plans/#price-adjustments)
        for details on the price adjustments.

        Args:
          starting_on: RFC 3339 timestamp for when the plan becomes active for this customer. Must be
              at 0:00 UTC (midnight).

          ending_before: RFC 3339 timestamp for when the plan ends (exclusive) for this customer. Must be
              at 0:00 UTC (midnight).

          net_payment_terms_days: Number of days after issuance of invoice after which the invoice is due (e.g.
              Net 30).

          overage_rate_adjustments: An optional list of overage rates that override the rates of the original plan
              configuration. These new rates will apply to all pricing ramps.

          price_adjustments: A list of price adjustments can be applied on top of the pricing in the plans.
              See the
              [price adjustments documentation](https://plans-docs.metronome.com/pricing/managing-plans/#price-adjustments)
              for details.

          trial_spec: A custom trial can be set for the customer's plan. See the
              [trial configuration documentation](https://docs.metronome.com/provisioning/configure-trials/)
              for details.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return await self._post(
            f"/v1/customers/{customer_id}/plans/add",
            body=await async_maybe_transform(
                {
                    "plan_id": plan_id,
                    "starting_on": starting_on,
                    "ending_before": ending_before,
                    "net_payment_terms_days": net_payment_terms_days,
                    "overage_rate_adjustments": overage_rate_adjustments,
                    "price_adjustments": price_adjustments,
                    "trial_spec": trial_spec,
                },
                plan_add_params.PlanAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanAddResponse,
        )

    async def end(
        self,
        *,
        customer_id: str,
        customer_plan_id: str,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        void_invoices: bool | NotGiven = NOT_GIVEN,
        void_stripe_invoices: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanEndResponse:
        """
        Change the end date of a customer's plan.

        Args:
          ending_before: RFC 3339 timestamp for when the plan ends (exclusive) for this customer. Must be
              at 0:00 UTC (midnight). If not provided, the plan end date will be cleared.

          void_invoices: If true, plan end date can be before the last finalized invoice date. Any
              invoices generated after the plan end date will be voided.

          void_stripe_invoices: Only applicable when void_invoices is set to true. If true, for every invoice
              that is voided we will also attempt to void/delete the stripe invoice (if any).
              Stripe invoices will be voided if finalized or deleted if still in draft state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not customer_plan_id:
            raise ValueError(f"Expected a non-empty value for `customer_plan_id` but received {customer_plan_id!r}")
        return await self._post(
            f"/v1/customers/{customer_id}/plans/{customer_plan_id}/end",
            body=await async_maybe_transform(
                {
                    "ending_before": ending_before,
                    "void_invoices": void_invoices,
                    "void_stripe_invoices": void_stripe_invoices,
                },
                plan_end_params.PlanEndParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanEndResponse,
        )

    def list_price_adjustments(
        self,
        *,
        customer_id: str,
        customer_plan_id: str,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PlanListPriceAdjustmentsResponse, AsyncCursorPage[PlanListPriceAdjustmentsResponse]]:
        """Lists a customer plans adjustments.

        See the
        [price adjustments documentation](https://plans-docs.metronome.com/pricing/managing-plans/#price-adjustments)
        for details.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not customer_plan_id:
            raise ValueError(f"Expected a non-empty value for `customer_plan_id` but received {customer_plan_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/plans/{customer_plan_id}/priceAdjustments",
            page=AsyncCursorPage[PlanListPriceAdjustmentsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_price_adjustments_params.PlanListPriceAdjustmentsParams,
                ),
            ),
            model=PlanListPriceAdjustmentsResponse,
        )


class PlansResourceWithRawResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.list = to_raw_response_wrapper(
            plans.list,
        )
        self.add = to_raw_response_wrapper(
            plans.add,
        )
        self.end = to_raw_response_wrapper(
            plans.end,
        )
        self.list_price_adjustments = to_raw_response_wrapper(
            plans.list_price_adjustments,
        )


class AsyncPlansResourceWithRawResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.list = async_to_raw_response_wrapper(
            plans.list,
        )
        self.add = async_to_raw_response_wrapper(
            plans.add,
        )
        self.end = async_to_raw_response_wrapper(
            plans.end,
        )
        self.list_price_adjustments = async_to_raw_response_wrapper(
            plans.list_price_adjustments,
        )


class PlansResourceWithStreamingResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.list = to_streamed_response_wrapper(
            plans.list,
        )
        self.add = to_streamed_response_wrapper(
            plans.add,
        )
        self.end = to_streamed_response_wrapper(
            plans.end,
        )
        self.list_price_adjustments = to_streamed_response_wrapper(
            plans.list_price_adjustments,
        )


class AsyncPlansResourceWithStreamingResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.list = async_to_streamed_response_wrapper(
            plans.list,
        )
        self.add = async_to_streamed_response_wrapper(
            plans.add,
        )
        self.end = async_to_streamed_response_wrapper(
            plans.end,
        )
        self.list_price_adjustments = async_to_streamed_response_wrapper(
            plans.list_price_adjustments,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.commit import Commit
from ..shared.credit import Credit
from ..shared.discount import Discount
from ..shared.override import Override
from ..shared.pro_service import ProService
from ..shared.scheduled_charge import ScheduledCharge
from ..shared.contract_without_amendments import ContractWithoutAmendments

__all__ = [
    "ContractRetrieveResponse",
    "Data",
    "DataAmendment",
    "DataAmendmentResellerRoyalty",
    "DataCustomerBillingProviderConfiguration",
    "DataPrepaidBalanceThresholdConfiguration",
    "DataPrepaidBalanceThresholdConfigurationCommit",
    "DataPrepaidBalanceThresholdConfigurationCommitSpecifier",
    "DataPrepaidBalanceThresholdConfigurationPaymentGateConfig",
    "DataPrepaidBalanceThresholdConfigurationPaymentGateConfigPrecalculatedTaxConfig",
    "DataPrepaidBalanceThresholdConfigurationPaymentGateConfigStripeConfig",
    "DataSpendThresholdConfiguration",
    "DataSpendThresholdConfigurationCommit",
    "DataSpendThresholdConfigurationPaymentGateConfig",
    "DataSpendThresholdConfigurationPaymentGateConfigPrecalculatedTaxConfig",
    "DataSpendThresholdConfigurationPaymentGateConfigStripeConfig",
    "DataSubscription",
    "DataSubscriptionProration",
    "DataSubscriptionQuantitySchedule",
    "DataSubscriptionSubscriptionRate",
    "DataSubscriptionSubscriptionRateProduct",
]


class DataAmendmentResellerRoyalty(BaseModel):
    reseller_type: Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]

    aws_account_number: Optional[str] = None

    aws_offer_id: Optional[str] = None

    aws_payer_reference_id: Optional[str] = None

    ending_before: Optional[datetime] = None

    fraction: Optional[float] = None

    gcp_account_id: Optional[str] = None

    gcp_offer_id: Optional[str] = None

    netsuite_reseller_id: Optional[str] = None

    reseller_contract_value: Optional[float] = None

    starting_at: Optional[datetime] = None


class DataAmendment(BaseModel):
    id: str

    commits: List[Commit]

    created_at: datetime

    created_by: str

    overrides: List[Override]

    scheduled_charges: List[ScheduledCharge]

    starting_at: datetime

    credits: Optional[List[Credit]] = None

    discounts: Optional[List[Discount]] = None
    """This field's availability is dependent on your client's configuration."""

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    professional_services: Optional[List[ProService]] = None
    """This field's availability is dependent on your client's configuration."""

    reseller_royalties: Optional[List[DataAmendmentResellerRoyalty]] = None
    """This field's availability is dependent on your client's configuration."""

    salesforce_opportunity_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""


class DataCustomerBillingProviderConfiguration(BaseModel):
    billing_provider: Literal[
        "aws_marketplace",
        "stripe",
        "netsuite",
        "custom",
        "azure_marketplace",
        "quickbooks_online",
        "workday",
        "gcp_marketplace",
    ]

    delivery_method: Literal["direct_to_billing_provider", "aws_sqs", "tackle", "aws_sns"]

    id: Optional[str] = None

    configuration: Optional[Dict[str, object]] = None
    """Configuration for the billing provider.

    The structure of this object is specific to the billing provider.
    """


class DataPrepaidBalanceThresholdConfigurationCommitSpecifier(BaseModel):
    presentation_group_values: Optional[Dict[str, str]] = None

    pricing_group_values: Optional[Dict[str, str]] = None

    product_id: Optional[str] = None
    """
    If provided, the specifier will only apply to the product with the specified ID.
    """

    product_tags: Optional[List[str]] = None
    """
    If provided, the specifier will only apply to products with all the specified
    tags.
    """


class DataPrepaidBalanceThresholdConfigurationCommit(BaseModel):
    product_id: str
    """
    The commit product that will be used to generate the line item for commit
    payment.
    """

    applicable_product_ids: Optional[List[str]] = None
    """Which products the threshold commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    applicable_product_tags: Optional[List[str]] = None
    """Which tags the threshold commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    description: Optional[str] = None

    name: Optional[str] = None
    """Specify the name of the line item for the threshold charge.

    If left blank, it will default to the commit product name.
    """

    specifiers: Optional[List[DataPrepaidBalanceThresholdConfigurationCommitSpecifier]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """


class DataPrepaidBalanceThresholdConfigurationPaymentGateConfigPrecalculatedTaxConfig(BaseModel):
    tax_amount: float
    """Amount of tax to be applied.

    This should be in the same currency and denomination as the commit's invoice
    schedule
    """

    tax_name: Optional[str] = None
    """Name of the tax to be applied.

    This may be used in an invoice line item description.
    """


class DataPrepaidBalanceThresholdConfigurationPaymentGateConfigStripeConfig(BaseModel):
    payment_type: Literal["INVOICE", "PAYMENT_INTENT"]
    """If left blank, will default to INVOICE"""

    invoice_metadata: Optional[Dict[str, str]] = None
    """Metadata to be added to the Stripe invoice.

    Only applicable if using INVOICE as your payment type.
    """


class DataPrepaidBalanceThresholdConfigurationPaymentGateConfig(BaseModel):
    payment_gate_type: Literal["NONE", "STRIPE", "EXTERNAL"]
    """Gate access to the commit balance based on successful collection of payment.

    Select STRIPE for Metronome to facilitate payment via Stripe. Select EXTERNAL to
    facilitate payment using your own payment integration. Select NONE if you do not
    wish to payment gate the commit balance.
    """

    precalculated_tax_config: Optional[
        DataPrepaidBalanceThresholdConfigurationPaymentGateConfigPrecalculatedTaxConfig
    ] = None
    """Only applicable if using PRECALCULATED as your tax type."""

    stripe_config: Optional[DataPrepaidBalanceThresholdConfigurationPaymentGateConfigStripeConfig] = None
    """Only applicable if using STRIPE as your payment gate type."""

    tax_type: Optional[Literal["NONE", "STRIPE", "ANROK", "PRECALCULATED"]] = None
    """Stripe tax is only supported for Stripe payment gateway.

    Select NONE if you do not wish Metronome to calculate tax on your behalf.
    Leaving this field blank will default to NONE.
    """


class DataPrepaidBalanceThresholdConfiguration(BaseModel):
    commit: DataPrepaidBalanceThresholdConfigurationCommit

    is_enabled: bool
    """
    When set to false, the contract will not be evaluated against the
    threshold_amount. Toggling to true will result an immediate evaluation,
    regardless of prior state.
    """

    payment_gate_config: DataPrepaidBalanceThresholdConfigurationPaymentGateConfig

    recharge_to_amount: float
    """Specify the amount the balance should be recharged to."""

    threshold_amount: float
    """Specify the threshold amount for the contract.

    Each time the contract's prepaid balance lowers to this amount, a threshold
    charge will be initiated.
    """

    custom_credit_type_id: Optional[str] = None
    """
    If provided, the threshold, recharge-to amount, and the resulting threshold
    commit amount will be in terms of this credit type instead of the fiat currency.
    """


class DataSpendThresholdConfigurationCommit(BaseModel):
    product_id: str
    """
    The commit product that will be used to generate the line item for commit
    payment.
    """

    description: Optional[str] = None

    name: Optional[str] = None
    """Specify the name of the line item for the threshold charge.

    If left blank, it will default to the commit product name.
    """


class DataSpendThresholdConfigurationPaymentGateConfigPrecalculatedTaxConfig(BaseModel):
    tax_amount: float
    """Amount of tax to be applied.

    This should be in the same currency and denomination as the commit's invoice
    schedule
    """

    tax_name: Optional[str] = None
    """Name of the tax to be applied.

    This may be used in an invoice line item description.
    """


class DataSpendThresholdConfigurationPaymentGateConfigStripeConfig(BaseModel):
    payment_type: Literal["INVOICE", "PAYMENT_INTENT"]
    """If left blank, will default to INVOICE"""

    invoice_metadata: Optional[Dict[str, str]] = None
    """Metadata to be added to the Stripe invoice.

    Only applicable if using INVOICE as your payment type.
    """


class DataSpendThresholdConfigurationPaymentGateConfig(BaseModel):
    payment_gate_type: Literal["NONE", "STRIPE", "EXTERNAL"]
    """Gate access to the commit balance based on successful collection of payment.

    Select STRIPE for Metronome to facilitate payment via Stripe. Select EXTERNAL to
    facilitate payment using your own payment integration. Select NONE if you do not
    wish to payment gate the commit balance.
    """

    precalculated_tax_config: Optional[DataSpendThresholdConfigurationPaymentGateConfigPrecalculatedTaxConfig] = None
    """Only applicable if using PRECALCULATED as your tax type."""

    stripe_config: Optional[DataSpendThresholdConfigurationPaymentGateConfigStripeConfig] = None
    """Only applicable if using STRIPE as your payment gate type."""

    tax_type: Optional[Literal["NONE", "STRIPE", "ANROK", "PRECALCULATED"]] = None
    """Stripe tax is only supported for Stripe payment gateway.

    Select NONE if you do not wish Metronome to calculate tax on your behalf.
    Leaving this field blank will default to NONE.
    """


class DataSpendThresholdConfiguration(BaseModel):
    commit: DataSpendThresholdConfigurationCommit

    is_enabled: bool
    """
    When set to false, the contract will not be evaluated against the
    threshold_amount. Toggling to true will result an immediate evaluation,
    regardless of prior state.
    """

    payment_gate_config: DataSpendThresholdConfigurationPaymentGateConfig

    threshold_amount: float
    """Specify the threshold amount for the contract.

    Each time the contract's usage hits this amount, a threshold charge will be
    initiated.
    """


class DataSubscriptionProration(BaseModel):
    invoice_behavior: Literal["BILL_IMMEDIATELY", "BILL_ON_NEXT_COLLECTION_DATE"]

    is_prorated: bool


class DataSubscriptionQuantitySchedule(BaseModel):
    quantity: float

    starting_at: datetime

    ending_before: Optional[datetime] = None


class DataSubscriptionSubscriptionRateProduct(BaseModel):
    id: str

    name: str


class DataSubscriptionSubscriptionRate(BaseModel):
    billing_frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]

    product: DataSubscriptionSubscriptionRateProduct


class DataSubscription(BaseModel):
    collection_schedule: Literal["ADVANCE", "ARREARS"]

    proration: DataSubscriptionProration

    quantity_schedule: List[DataSubscriptionQuantitySchedule]
    """List of quantity schedule items for the subscription.

    Only includes the current quantity and future quantity changes.
    """

    starting_at: datetime

    subscription_rate: DataSubscriptionSubscriptionRate

    id: Optional[str] = None

    custom_fields: Optional[Dict[str, str]] = None

    description: Optional[str] = None

    ending_before: Optional[datetime] = None

    fiat_credit_type_id: Optional[str] = None

    name: Optional[str] = None


class Data(BaseModel):
    id: str

    amendments: List[DataAmendment]

    current: ContractWithoutAmendments

    customer_id: str

    initial: ContractWithoutAmendments

    archived_at: Optional[datetime] = None
    """RFC 3339 timestamp indicating when the contract was archived.

    If not returned, the contract is not archived.
    """

    custom_fields: Optional[Dict[str, str]] = None

    customer_billing_provider_configuration: Optional[DataCustomerBillingProviderConfiguration] = None
    """The billing provider configuration associated with a contract."""

    prepaid_balance_threshold_configuration: Optional[DataPrepaidBalanceThresholdConfiguration] = None

    priority: Optional[float] = None
    """Priority of the contract."""

    scheduled_charges_on_usage_invoices: Optional[Literal["ALL"]] = None
    """
    Determines which scheduled and commit charges to consolidate onto the Contract's
    usage invoice. The charge's `timestamp` must match the usage invoice's
    `ending_before` date for consolidation to occur. This field cannot be modified
    after a Contract has been created. If this field is omitted, charges will appear
    on a separate invoice from usage charges.
    """

    spend_threshold_configuration: Optional[DataSpendThresholdConfiguration] = None

    subscriptions: Optional[List[DataSubscription]] = None
    """List of subscriptions on the contract."""

    uniqueness_key: Optional[str] = None
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """


class ContractRetrieveResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import alert_create_params, alert_archive_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.alert_create_response import AlertCreateResponse
from ...types.v1.alert_archive_response import AlertArchiveResponse

__all__ = ["AlertsResource", "AsyncAlertsResource"]


class AlertsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AlertsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AlertsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AlertsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AlertsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        alert_type: Literal[
            "low_credit_balance_reached",
            "spend_threshold_reached",
            "monthly_invoice_total_spend_threshold_reached",
            "low_remaining_days_in_plan_reached",
            "low_remaining_credit_percentage_reached",
            "usage_threshold_reached",
            "low_remaining_days_for_commit_segment_reached",
            "low_remaining_commit_balance_reached",
            "low_remaining_commit_percentage_reached",
            "low_remaining_days_for_contract_credit_segment_reached",
            "low_remaining_contract_credit_balance_reached",
            "low_remaining_contract_credit_percentage_reached",
            "low_remaining_contract_credit_and_commit_balance_reached",
            "invoice_total_reached",
        ],
        name: str,
        threshold: float,
        billable_metric_id: str | NotGiven = NOT_GIVEN,
        credit_grant_type_filters: List[str] | NotGiven = NOT_GIVEN,
        credit_type_id: str | NotGiven = NOT_GIVEN,
        custom_field_filters: Iterable[alert_create_params.CustomFieldFilter] | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        evaluate_on_create: bool | NotGiven = NOT_GIVEN,
        group_values: Iterable[alert_create_params.GroupValue] | NotGiven = NOT_GIVEN,
        invoice_types_filter: List[str] | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        uniqueness_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AlertCreateResponse:
        """
        Create a new alert

        Args:
          alert_type: Type of the alert

          name: Name of the alert

          threshold: Threshold value of the alert policy. Depending upon the alert type, this number
              may represent a financial amount, the days remaining, or a percentage reached.

          billable_metric_id: For alerts of type `usage_threshold_reached`, specifies which billable metric to
              track the usage for.

          credit_grant_type_filters: An array of strings, representing a way to filter the credit grant this alert
              applies to, by looking at the credit_grant_type field on the credit grant. This
              field is only defined for CreditPercentage and CreditBalance alerts

          credit_type_id: ID of the credit's currency, defaults to USD. If the specific alert type
              requires a pricing unit/currency, find the ID in the
              [Metronome app](https://app.metronome.com/offering/pricing-units).

          custom_field_filters: A list of custom field filters for alert types that support advanced filtering.
              Only present for contract invoices.

          customer_id: If provided, will create this alert for this specific customer. To create an
              alert for all customers, do not specify a `customer_id`.

          evaluate_on_create: If true, the alert will evaluate immediately on customers that already meet the
              alert threshold. If false, it will only evaluate on future customers that
              trigger the alert threshold. Defaults to true.

          group_values: Only present for `spend_threshold_reached` alerts. Scope alert to a specific
              group key on individual line items.

          invoice_types_filter: Only supported for invoice_total_reached alerts. A list of invoice types to
              evaluate.

          plan_id: If provided, will create this alert for this specific plan. To create an alert
              for all customers, do not specify a `plan_id`.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a record is made
              with a previously used uniqueness key, a new record will not be created and the
              request will fail with a 409 error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/alerts/create",
            body=maybe_transform(
                {
                    "alert_type": alert_type,
                    "name": name,
                    "threshold": threshold,
                    "billable_metric_id": billable_metric_id,
                    "credit_grant_type_filters": credit_grant_type_filters,
                    "credit_type_id": credit_type_id,
                    "custom_field_filters": custom_field_filters,
                    "customer_id": customer_id,
                    "evaluate_on_create": evaluate_on_create,
                    "group_values": group_values,
                    "invoice_types_filter": invoice_types_filter,
                    "plan_id": plan_id,
                    "uniqueness_key": uniqueness_key,
                },
                alert_create_params.AlertCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertCreateResponse,
        )

    def archive(
        self,
        *,
        id: str,
        release_uniqueness_key: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AlertArchiveResponse:
        """
        Archive an existing alert

        Args:
          id: The Metronome ID of the alert

          release_uniqueness_key: If true, resets the uniqueness key on this alert so it can be re-used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/alerts/archive",
            body=maybe_transform(
                {
                    "id": id,
                    "release_uniqueness_key": release_uniqueness_key,
                },
                alert_archive_params.AlertArchiveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertArchiveResponse,
        )


class AsyncAlertsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAlertsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAlertsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAlertsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncAlertsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        alert_type: Literal[
            "low_credit_balance_reached",
            "spend_threshold_reached",
            "monthly_invoice_total_spend_threshold_reached",
            "low_remaining_days_in_plan_reached",
            "low_remaining_credit_percentage_reached",
            "usage_threshold_reached",
            "low_remaining_days_for_commit_segment_reached",
            "low_remaining_commit_balance_reached",
            "low_remaining_commit_percentage_reached",
            "low_remaining_days_for_contract_credit_segment_reached",
            "low_remaining_contract_credit_balance_reached",
            "low_remaining_contract_credit_percentage_reached",
            "low_remaining_contract_credit_and_commit_balance_reached",
            "invoice_total_reached",
        ],
        name: str,
        threshold: float,
        billable_metric_id: str | NotGiven = NOT_GIVEN,
        credit_grant_type_filters: List[str] | NotGiven = NOT_GIVEN,
        credit_type_id: str | NotGiven = NOT_GIVEN,
        custom_field_filters: Iterable[alert_create_params.CustomFieldFilter] | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        evaluate_on_create: bool | NotGiven = NOT_GIVEN,
        group_values: Iterable[alert_create_params.GroupValue] | NotGiven = NOT_GIVEN,
        invoice_types_filter: List[str] | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        uniqueness_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AlertCreateResponse:
        """
        Create a new alert

        Args:
          alert_type: Type of the alert

          name: Name of the alert

          threshold: Threshold value of the alert policy. Depending upon the alert type, this number
              may represent a financial amount, the days remaining, or a percentage reached.

          billable_metric_id: For alerts of type `usage_threshold_reached`, specifies which billable metric to
              track the usage for.

          credit_grant_type_filters: An array of strings, representing a way to filter the credit grant this alert
              applies to, by looking at the credit_grant_type field on the credit grant. This
              field is only defined for CreditPercentage and CreditBalance alerts

          credit_type_id: ID of the credit's currency, defaults to USD. If the specific alert type
              requires a pricing unit/currency, find the ID in the
              [Metronome app](https://app.metronome.com/offering/pricing-units).

          custom_field_filters: A list of custom field filters for alert types that support advanced filtering.
              Only present for contract invoices.

          customer_id: If provided, will create this alert for this specific customer. To create an
              alert for all customers, do not specify a `customer_id`.

          evaluate_on_create: If true, the alert will evaluate immediately on customers that already meet the
              alert threshold. If false, it will only evaluate on future customers that
              trigger the alert threshold. Defaults to true.

          group_values: Only present for `spend_threshold_reached` alerts. Scope alert to a specific
              group key on individual line items.

          invoice_types_filter: Only supported for invoice_total_reached alerts. A list of invoice types to
              evaluate.

          plan_id: If provided, will create this alert for this specific plan. To create an alert
              for all customers, do not specify a `plan_id`.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a record is made
              with a previously used uniqueness key, a new record will not be created and the
              request will fail with a 409 error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/alerts/create",
            body=await async_maybe_transform(
                {
                    "alert_type": alert_type,
                    "name": name,
                    "threshold": threshold,
                    "billable_metric_id": billable_metric_id,
                    "credit_grant_type_filters": credit_grant_type_filters,
                    "credit_type_id": credit_type_id,
                    "custom_field_filters": custom_field_filters,
                    "customer_id": customer_id,
                    "evaluate_on_create": evaluate_on_create,
                    "group_values": group_values,
                    "invoice_types_filter": invoice_types_filter,
                    "plan_id": plan_id,
                    "uniqueness_key": uniqueness_key,
                },
                alert_create_params.AlertCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertCreateResponse,
        )

    async def archive(
        self,
        *,
        id: str,
        release_uniqueness_key: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AlertArchiveResponse:
        """
        Archive an existing alert

        Args:
          id: The Metronome ID of the alert

          release_uniqueness_key: If true, resets the uniqueness key on this alert so it can be re-used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/alerts/archive",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "release_uniqueness_key": release_uniqueness_key,
                },
                alert_archive_params.AlertArchiveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertArchiveResponse,
        )


class AlertsResourceWithRawResponse:
    def __init__(self, alerts: AlertsResource) -> None:
        self._alerts = alerts

        self.create = to_raw_response_wrapper(
            alerts.create,
        )
        self.archive = to_raw_response_wrapper(
            alerts.archive,
        )


class AsyncAlertsResourceWithRawResponse:
    def __init__(self, alerts: AsyncAlertsResource) -> None:
        self._alerts = alerts

        self.create = async_to_raw_response_wrapper(
            alerts.create,
        )
        self.archive = async_to_raw_response_wrapper(
            alerts.archive,
        )


class AlertsResourceWithStreamingResponse:
    def __init__(self, alerts: AlertsResource) -> None:
        self._alerts = alerts

        self.create = to_streamed_response_wrapper(
            alerts.create,
        )
        self.archive = to_streamed_response_wrapper(
            alerts.archive,
        )


class AsyncAlertsResourceWithStreamingResponse:
    def __init__(self, alerts: AsyncAlertsResource) -> None:
        self._alerts = alerts

        self.create = async_to_streamed_response_wrapper(
            alerts.create,
        )
        self.archive = async_to_streamed_response_wrapper(
            alerts.archive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["QuantityConversion"]


class QuantityConversion(BaseModel):
    conversion_factor: float
    """The factor to multiply or divide the quantity by."""

    operation: Literal["MULTIPLY", "DIVIDE"]
    """The operation to perform on the quantity"""

    name: Optional[str] = None
    """Optional name for this conversion."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime

import httpx

from .rates import (
    RatesResource,
    AsyncRatesResource,
    RatesResourceWithRawResponse,
    AsyncRatesResourceWithRawResponse,
    RatesResourceWithStreamingResponse,
    AsyncRatesResourceWithStreamingResponse,
)
from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncCursorPage, AsyncCursorPage
from .product_orders import (
    ProductOrdersResource,
    AsyncProductOrdersResource,
    ProductOrdersResourceWithRawResponse,
    AsyncProductOrdersResourceWithRawResponse,
    ProductOrdersResourceWithStreamingResponse,
    AsyncProductOrdersResourceWithStreamingResponse,
)
from .named_schedules import (
    NamedSchedulesResource,
    AsyncNamedSchedulesResource,
    NamedSchedulesResourceWithRawResponse,
    AsyncNamedSchedulesResourceWithRawResponse,
    NamedSchedulesResourceWithStreamingResponse,
    AsyncNamedSchedulesResourceWithStreamingResponse,
)
from ....._base_client import AsyncPaginator, make_request_options
from .....types.v1.contracts import (
    rate_card_list_params,
    rate_card_create_params,
    rate_card_update_params,
    rate_card_archive_params,
    rate_card_retrieve_params,
    rate_card_retrieve_rate_schedule_params,
)
from .....types.v1.contracts.rate_card_list_response import RateCardListResponse
from .....types.v1.contracts.rate_card_create_response import RateCardCreateResponse
from .....types.v1.contracts.rate_card_update_response import RateCardUpdateResponse
from .....types.v1.contracts.rate_card_archive_response import RateCardArchiveResponse
from .....types.v1.contracts.rate_card_retrieve_response import RateCardRetrieveResponse
from .....types.v1.contracts.rate_card_retrieve_rate_schedule_response import RateCardRetrieveRateScheduleResponse

__all__ = ["RateCardsResource", "AsyncRateCardsResource"]


class RateCardsResource(SyncAPIResource):
    @cached_property
    def product_orders(self) -> ProductOrdersResource:
        return ProductOrdersResource(self._client)

    @cached_property
    def rates(self) -> RatesResource:
        return RatesResource(self._client)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResource:
        return NamedSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> RateCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return RateCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RateCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return RateCardsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        aliases: Iterable[rate_card_create_params.Alias] | NotGiven = NOT_GIVEN,
        credit_type_conversions: Iterable[rate_card_create_params.CreditTypeConversion] | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, str] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        fiat_credit_type_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardCreateResponse:
        """Create a new rate card

        Args:
          name: Used only in UI/API.

        It is not exposed to end customers.

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          credit_type_conversions: Required when using custom pricing units in rates.

          fiat_credit_type_id: The Metronome ID of the credit type to associate with the rate card, defaults to
              USD (cents) if not passed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/create",
            body=maybe_transform(
                {
                    "name": name,
                    "aliases": aliases,
                    "credit_type_conversions": credit_type_conversions,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fiat_credit_type_id": fiat_credit_type_id,
                },
                rate_card_create_params.RateCardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardCreateResponse,
        )

    def retrieve(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardRetrieveResponse:
        """
        Get a specific rate card NOTE: Use `/contract-pricing/rate-cards/getRates` to
        retrieve rate card rates.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/get",
            body=maybe_transform({"id": id}, rate_card_retrieve_params.RateCardRetrieveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardRetrieveResponse,
        )

    def update(
        self,
        *,
        rate_card_id: str,
        aliases: Iterable[rate_card_update_params.Alias] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardUpdateResponse:
        """
        Update a rate card

        Args:
          rate_card_id: ID of the rate card to update

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          name: Used only in UI/API. It is not exposed to end customers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/update",
            body=maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "aliases": aliases,
                    "description": description,
                    "name": name,
                },
                rate_card_update_params.RateCardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardUpdateResponse,
        )

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        body: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[RateCardListResponse]:
        """
        List rate cards NOTE: Use `/contract-pricing/rate-cards/getRates` to retrieve
        rate card rates.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contract-pricing/rate-cards/list",
            page=SyncCursorPage[RateCardListResponse],
            body=maybe_transform(body, rate_card_list_params.RateCardListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_list_params.RateCardListParams,
                ),
            ),
            model=RateCardListResponse,
            method="post",
        )

    def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardArchiveResponse:
        """
        Archive a rate card

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/archive",
            body=maybe_transform({"id": id}, rate_card_archive_params.RateCardArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardArchiveResponse,
        )

    def retrieve_rate_schedule(
        self,
        *,
        rate_card_id: str,
        starting_at: Union[str, datetime],
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        selectors: Iterable[rate_card_retrieve_rate_schedule_params.Selector] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardRetrieveRateScheduleResponse:
        """
        Get all rates for a rate card from starting_at (either in perpetuity or until
        ending_before, if provided)

        Args:
          rate_card_id: ID of the rate card to get the schedule for

          starting_at: inclusive starting point for the rates schedule

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          ending_before: optional exclusive end date for the rates schedule. When not specified rates
              will show all future schedule segments.

          selectors: List of rate selectors, rates matching ANY of the selector will be included in
              the response Passing no selectors will result in all rates being returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/getRateSchedule",
            body=maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "starting_at": starting_at,
                    "ending_before": ending_before,
                    "selectors": selectors,
                },
                rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
                ),
            ),
            cast_to=RateCardRetrieveRateScheduleResponse,
        )


class AsyncRateCardsResource(AsyncAPIResource):
    @cached_property
    def product_orders(self) -> AsyncProductOrdersResource:
        return AsyncProductOrdersResource(self._client)

    @cached_property
    def rates(self) -> AsyncRatesResource:
        return AsyncRatesResource(self._client)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResource:
        return AsyncNamedSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRateCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRateCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRateCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncRateCardsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        aliases: Iterable[rate_card_create_params.Alias] | NotGiven = NOT_GIVEN,
        credit_type_conversions: Iterable[rate_card_create_params.CreditTypeConversion] | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, str] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        fiat_credit_type_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardCreateResponse:
        """Create a new rate card

        Args:
          name: Used only in UI/API.

        It is not exposed to end customers.

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          credit_type_conversions: Required when using custom pricing units in rates.

          fiat_credit_type_id: The Metronome ID of the credit type to associate with the rate card, defaults to
              USD (cents) if not passed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/create",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "aliases": aliases,
                    "credit_type_conversions": credit_type_conversions,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fiat_credit_type_id": fiat_credit_type_id,
                },
                rate_card_create_params.RateCardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardCreateResponse,
        )

    async def retrieve(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardRetrieveResponse:
        """
        Get a specific rate card NOTE: Use `/contract-pricing/rate-cards/getRates` to
        retrieve rate card rates.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/get",
            body=await async_maybe_transform({"id": id}, rate_card_retrieve_params.RateCardRetrieveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardRetrieveResponse,
        )

    async def update(
        self,
        *,
        rate_card_id: str,
        aliases: Iterable[rate_card_update_params.Alias] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardUpdateResponse:
        """
        Update a rate card

        Args:
          rate_card_id: ID of the rate card to update

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          name: Used only in UI/API. It is not exposed to end customers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/update",
            body=await async_maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "aliases": aliases,
                    "description": description,
                    "name": name,
                },
                rate_card_update_params.RateCardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardUpdateResponse,
        )

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        body: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[RateCardListResponse, AsyncCursorPage[RateCardListResponse]]:
        """
        List rate cards NOTE: Use `/contract-pricing/rate-cards/getRates` to retrieve
        rate card rates.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contract-pricing/rate-cards/list",
            page=AsyncCursorPage[RateCardListResponse],
            body=maybe_transform(body, rate_card_list_params.RateCardListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_list_params.RateCardListParams,
                ),
            ),
            model=RateCardListResponse,
            method="post",
        )

    async def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardArchiveResponse:
        """
        Archive a rate card

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/archive",
            body=await async_maybe_transform({"id": id}, rate_card_archive_params.RateCardArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardArchiveResponse,
        )

    async def retrieve_rate_schedule(
        self,
        *,
        rate_card_id: str,
        starting_at: Union[str, datetime],
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        selectors: Iterable[rate_card_retrieve_rate_schedule_params.Selector] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateCardRetrieveRateScheduleResponse:
        """
        Get all rates for a rate card from starting_at (either in perpetuity or until
        ending_before, if provided)

        Args:
          rate_card_id: ID of the rate card to get the schedule for

          starting_at: inclusive starting point for the rates schedule

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          ending_before: optional exclusive end date for the rates schedule. When not specified rates
              will show all future schedule segments.

          selectors: List of rate selectors, rates matching ANY of the selector will be included in
              the response Passing no selectors will result in all rates being returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/getRateSchedule",
            body=await async_maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "starting_at": starting_at,
                    "ending_before": ending_before,
                    "selectors": selectors,
                },
                rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
                ),
            ),
            cast_to=RateCardRetrieveRateScheduleResponse,
        )


class RateCardsResourceWithRawResponse:
    def __init__(self, rate_cards: RateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = to_raw_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = to_raw_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = to_raw_response_wrapper(
            rate_cards.update,
        )
        self.list = to_raw_response_wrapper(
            rate_cards.list,
        )
        self.archive = to_raw_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = to_raw_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> ProductOrdersResourceWithRawResponse:
        return ProductOrdersResourceWithRawResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> RatesResourceWithRawResponse:
        return RatesResourceWithRawResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResourceWithRawResponse:
        return NamedSchedulesResourceWithRawResponse(self._rate_cards.named_schedules)


class AsyncRateCardsResourceWithRawResponse:
    def __init__(self, rate_cards: AsyncRateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = async_to_raw_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            rate_cards.update,
        )
        self.list = async_to_raw_response_wrapper(
            rate_cards.list,
        )
        self.archive = async_to_raw_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = async_to_raw_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> AsyncProductOrdersResourceWithRawResponse:
        return AsyncProductOrdersResourceWithRawResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> AsyncRatesResourceWithRawResponse:
        return AsyncRatesResourceWithRawResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResourceWithRawResponse:
        return AsyncNamedSchedulesResourceWithRawResponse(self._rate_cards.named_schedules)


class RateCardsResourceWithStreamingResponse:
    def __init__(self, rate_cards: RateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = to_streamed_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            rate_cards.update,
        )
        self.list = to_streamed_response_wrapper(
            rate_cards.list,
        )
        self.archive = to_streamed_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = to_streamed_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> ProductOrdersResourceWithStreamingResponse:
        return ProductOrdersResourceWithStreamingResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> RatesResourceWithStreamingResponse:
        return RatesResourceWithStreamingResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResourceWithStreamingResponse:
        return NamedSchedulesResourceWithStreamingResponse(self._rate_cards.named_schedules)


class AsyncRateCardsResourceWithStreamingResponse:
    def __init__(self, rate_cards: AsyncRateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = async_to_streamed_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            rate_cards.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rate_cards.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = async_to_streamed_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> AsyncProductOrdersResourceWithStreamingResponse:
        return AsyncProductOrdersResourceWithStreamingResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> AsyncRatesResourceWithStreamingResponse:
        return AsyncRatesResourceWithStreamingResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResourceWithStreamingResponse:
        return AsyncNamedSchedulesResourceWithStreamingResponse(self._rate_cards.named_schedules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ServiceListResponse", "Service"]


class Service(BaseModel):
    ips: List[str]

    name: str

    usage: Literal["makes_connections_from", "accepts_connections_at"]


class ServiceListResponse(BaseModel):
    services: List[Service]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel
from .quantity_rounding import QuantityRounding
from .quantity_conversion import QuantityConversion
from .product_list_item_state import ProductListItemState

__all__ = ["ProductListResponse", "Update"]


class Update(BaseModel):
    created_at: datetime

    created_by: str

    billable_metric_id: Optional[str] = None

    composite_product_ids: Optional[List[str]] = None

    composite_tags: Optional[List[str]] = None

    exclude_free_usage: Optional[bool] = None

    is_refundable: Optional[bool] = None

    name: Optional[str] = None

    netsuite_internal_item_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    netsuite_overage_item_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    presentation_group_key: Optional[List[str]] = None
    """For USAGE products only.

    Groups usage line items on invoices. The superset of values in the pricing group
    key and presentation group key must be set as one compound group key on the
    billable metric.
    """

    pricing_group_key: Optional[List[str]] = None
    """For USAGE products only.

    If set, pricing for this product will be determined for each pricing_group_key
    value, as opposed to the product as a whole. The superset of values in the
    pricing group key and presentation group key must be set as one compound group
    key on the billable metric.
    """

    quantity_conversion: Optional[QuantityConversion] = None
    """Optional.

    Only valid for USAGE products. If provided, the quantity will be converted using
    the provided conversion factor and operation. For example, if the operation is
    "multiply" and the conversion factor is 100, then the quantity will be
    multiplied by 100. This can be used in cases where data is sent in one unit and
    priced in another. For example, data could be sent in MB and priced in GB. In
    this case, the conversion factor would be 1024 and the operation would be
    "divide".
    """

    quantity_rounding: Optional[QuantityRounding] = None
    """Optional.

    Only valid for USAGE products. If provided, the quantity will be rounded using
    the provided rounding method and decimal places. For example, if the method is
    "round up" and the decimal places is 0, then the quantity will be rounded up to
    the nearest integer.
    """

    starting_at: Optional[datetime] = None

    tags: Optional[List[str]] = None


class ProductListResponse(BaseModel):
    id: str

    current: ProductListItemState

    initial: ProductListItemState

    type: Literal["USAGE", "SUBSCRIPTION", "COMPOSITE", "FIXED", "PRO_SERVICE"]

    updates: List[Update]

    archived_at: Optional[datetime] = None

    custom_fields: Optional[Dict[str, str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AlertResetParams"]


class AlertResetParams(TypedDict, total=False):
    alert_id: Required[str]
    """The Metronome ID of the alert"""

    customer_id: Required[str]
    """The Metronome ID of the customer"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["QuantityRoundingParam"]


class QuantityRoundingParam(TypedDict, total=False):
    decimal_places: Required[float]

    rounding_method: Required[Literal["ROUND_UP", "ROUND_DOWN", "ROUND_HALF_UP"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "ContractEditCreditParams",
    "AccessSchedule",
    "AccessScheduleAddScheduleItem",
    "AccessScheduleRemoveScheduleItem",
    "AccessScheduleUpdateScheduleItem",
    "Specifier",
]


class ContractEditCreditParams(TypedDict, total=False):
    credit_id: Required[str]
    """ID of the credit to edit"""

    customer_id: Required[str]
    """ID of the customer whose credit is being edited"""

    access_schedule: AccessSchedule

    applicable_product_ids: Optional[List[str]]
    """Which products the credit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    credit applies to all products.
    """

    applicable_product_tags: Optional[List[str]]
    """Which tags the credit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    credit applies to all products.
    """

    product_id: str

    specifiers: Optional[Iterable[Specifier]]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    Instead, to target usage by product or product tag, pass those values in the
    body of `specifiers`.
    """


class AccessScheduleAddScheduleItem(TypedDict, total=False):
    amount: Required[float]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]


class AccessScheduleRemoveScheduleItem(TypedDict, total=False):
    id: Required[str]


class AccessScheduleUpdateScheduleItem(TypedDict, total=False):
    id: Required[str]

    amount: float

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]


class AccessSchedule(TypedDict, total=False):
    add_schedule_items: Iterable[AccessScheduleAddScheduleItem]

    remove_schedule_items: Iterable[AccessScheduleRemoveScheduleItem]

    update_schedule_items: Iterable[AccessScheduleUpdateScheduleItem]


class Specifier(TypedDict, total=False):
    presentation_group_values: Dict[str, str]

    pricing_group_values: Dict[str, str]

    product_id: str
    """
    If provided, the specifier will only apply to the product with the specified ID.
    """

    product_tags: List[str]
    """
    If provided, the specifier will only apply to products with all the specified
    tags.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import TypedDict

__all__ = ["CustomerListParams"]


class CustomerListParams(TypedDict, total=False):
    customer_ids: List[str]
    """Filter the customer list by customer_id. Up to 100 ids can be provided."""

    ingest_alias: str
    """Filter the customer list by ingest_alias"""

    limit: int
    """Max number of results that should be returned"""

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    only_archived: bool
    """Filter the customer list to only return archived customers.

    By default, only active customers are returned.
    """

    salesforce_account_ids: List[str]
    """Filter the customer list by salesforce_account_id.

    Up to 100 ids can be provided.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .customer_alert import CustomerAlert

__all__ = ["AlertRetrieveResponse"]


class AlertRetrieveResponse(BaseModel):
    data: CustomerAlert



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from ...shared.commit import Commit

__all__ = ["CommitListResponse"]


class CommitListResponse(BaseModel):
    data: List[Commit]

    next_page: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypeAlias, TypedDict

from ..._utils import PropertyInfo
from .rollover_amount_max_amount_param import RolloverAmountMaxAmountParam
from .rollover_amount_max_percentage_param import RolloverAmountMaxPercentageParam

__all__ = ["CreditGrantCreateParams", "GrantAmount", "PaidAmount", "RolloverSettings", "RolloverSettingsRolloverAmount"]


class CreditGrantCreateParams(TypedDict, total=False):
    customer_id: Required[str]
    """the Metronome ID of the customer"""

    expires_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """
    The credit grant will only apply to usage or charges dated before this timestamp
    """

    grant_amount: Required[GrantAmount]
    """the amount of credits granted"""

    name: Required[str]
    """the name of the credit grant as it will appear on invoices"""

    paid_amount: Required[PaidAmount]
    """the amount paid for this credit grant"""

    priority: Required[float]

    credit_grant_type: str

    custom_fields: Dict[str, str]
    """Custom fields to attach to the credit grant."""

    effective_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    The credit grant will only apply to usage or charges dated on or after this
    timestamp
    """

    invoice_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The date to issue an invoice for the paid_amount."""

    product_ids: List[str]
    """The product(s) which these credits will be applied to.

    (If unspecified, the credits will be applied to charges for all products.). The
    array ordering specified here will be used to determine the order in which
    credits will be applied to invoice line items
    """

    reason: str

    rollover_settings: RolloverSettings
    """
    Configure a rollover for this credit grant so if it expires it rolls over a
    configured amount to a new credit grant. This feature is currently opt-in only.
    Contact Metronome to be added to the beta.
    """

    uniqueness_key: str
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """


class GrantAmount(TypedDict, total=False):
    amount: Required[float]

    credit_type_id: Required[str]
    """the ID of the pricing unit to be used. Defaults to USD (cents) if not passed."""


class PaidAmount(TypedDict, total=False):
    amount: Required[float]

    credit_type_id: Required[str]
    """the ID of the pricing unit to be used. Defaults to USD (cents) if not passed."""


RolloverSettingsRolloverAmount: TypeAlias = Union[RolloverAmountMaxPercentageParam, RolloverAmountMaxAmountParam]


class RolloverSettings(TypedDict, total=False):
    expires_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The date to expire the rollover credits."""

    priority: Required[float]
    """
    The priority to give the rollover credit grant that gets created when a rollover
    happens.
    """

    rollover_amount: Required[RolloverSettingsRolloverAmount]
    """Specify how much to rollover to the rollover credit grant"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorPage, AsyncCursorPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.v1.customers import (
    invoice_list_params,
    invoice_retrieve_params,
    invoice_add_charge_params,
    invoice_list_breakdowns_params,
)
from ....types.v1.customers.invoice import Invoice
from ....types.v1.customers.invoice_retrieve_response import InvoiceRetrieveResponse
from ....types.v1.customers.invoice_add_charge_response import InvoiceAddChargeResponse
from ....types.v1.customers.invoice_list_breakdowns_response import InvoiceListBreakdownsResponse

__all__ = ["InvoicesResource", "AsyncInvoicesResource"]


class InvoicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return InvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return InvoicesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        skip_zero_qty_line_items: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvoiceRetrieveResponse:
        """
        Fetch a specific invoice for a given customer.

        Args:
          skip_zero_qty_line_items: If set, all zero quantity line items will be filtered out of the response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not invoice_id:
            raise ValueError(f"Expected a non-empty value for `invoice_id` but received {invoice_id!r}")
        return self._get(
            f"/v1/customers/{customer_id}/invoices/{invoice_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"skip_zero_qty_line_items": skip_zero_qty_line_items},
                    invoice_retrieve_params.InvoiceRetrieveParams,
                ),
            ),
            cast_to=InvoiceRetrieveResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        credit_type_id: str | NotGiven = NOT_GIVEN,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        skip_zero_qty_line_items: bool | NotGiven = NOT_GIVEN,
        sort: Literal["date_asc", "date_desc"] | NotGiven = NOT_GIVEN,
        starting_on: Union[str, datetime] | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[Invoice]:
        """
        List all invoices for a given customer, optionally filtered by status, date
        range, and/or credit type.

        Args:
          credit_type_id: Only return invoices for the specified credit type

          ending_before: RFC 3339 timestamp (exclusive). Invoices will only be returned for billing
              periods that end before this time.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          skip_zero_qty_line_items: If set, all zero quantity line items will be filtered out of the response

          sort: Invoice sort order by issued_at, e.g. date_asc or date_desc. Defaults to
              date_asc.

          starting_on: RFC 3339 timestamp (inclusive). Invoices will only be returned for billing
              periods that start at or after this time.

          status: Invoice status, e.g. DRAFT, FINALIZED, or VOID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/invoices",
            page=SyncCursorPage[Invoice],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "credit_type_id": credit_type_id,
                        "ending_before": ending_before,
                        "limit": limit,
                        "next_page": next_page,
                        "skip_zero_qty_line_items": skip_zero_qty_line_items,
                        "sort": sort,
                        "starting_on": starting_on,
                        "status": status,
                    },
                    invoice_list_params.InvoiceListParams,
                ),
            ),
            model=Invoice,
        )

    def add_charge(
        self,
        *,
        customer_id: str,
        charge_id: str,
        customer_plan_id: str,
        description: str,
        invoice_start_timestamp: Union[str, datetime],
        price: float,
        quantity: float,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvoiceAddChargeResponse:
        """
        Add a one time charge to the specified invoice

        Args:
          charge_id: The Metronome ID of the charge to add to the invoice. Note that the charge must
              be on a product that is not on the current plan, and the product must have only
              fixed charges.

          customer_plan_id: The Metronome ID of the customer plan to add the charge to.

          invoice_start_timestamp: The start_timestamp of the invoice to add the charge to.

          price: The price of the charge. This price will match the currency on the invoice, e.g.
              USD cents.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._post(
            f"/v1/customers/{customer_id}/addCharge",
            body=maybe_transform(
                {
                    "charge_id": charge_id,
                    "customer_plan_id": customer_plan_id,
                    "description": description,
                    "invoice_start_timestamp": invoice_start_timestamp,
                    "price": price,
                    "quantity": quantity,
                },
                invoice_add_charge_params.InvoiceAddChargeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceAddChargeResponse,
        )

    def list_breakdowns(
        self,
        *,
        customer_id: str,
        ending_before: Union[str, datetime],
        starting_on: Union[str, datetime],
        credit_type_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        skip_zero_qty_line_items: bool | NotGiven = NOT_GIVEN,
        sort: Literal["date_asc", "date_desc"] | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        window_size: Literal["HOUR", "DAY"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[InvoiceListBreakdownsResponse]:
        """
        List daily or hourly invoice breakdowns for a given customer, optionally
        filtered by status, date range, and/or credit type. Important considerations:

        - If we receive backdated usage after an invoice has been finalized, the
          backdated usage will be included in the response and usage numbers may differ.

        Args:
          ending_before: RFC 3339 timestamp. Breakdowns will only be returned for time windows that end
              on or before this time.

          starting_on: RFC 3339 timestamp. Breakdowns will only be returned for time windows that start
              on or after this time.

          credit_type_id: Only return invoices for the specified credit type

          limit: Max number of results that should be returned. For daily breakdowns, the
              response can return up to 35 days worth of breakdowns. For hourly breakdowns,
              the response can return up to 24 hours. If there are more results, a cursor to
              the next page is returned.

          next_page: Cursor that indicates where the next page of results should start.

          skip_zero_qty_line_items: If set, all zero quantity line items will be filtered out of the response

          sort: Invoice sort order by issued_at, e.g. date_asc or date_desc. Defaults to
              date_asc.

          status: Invoice status, e.g. DRAFT or FINALIZED

          window_size: The granularity of the breakdowns to return. Defaults to day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/invoices/breakdowns",
            page=SyncCursorPage[InvoiceListBreakdownsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "starting_on": starting_on,
                        "credit_type_id": credit_type_id,
                        "limit": limit,
                        "next_page": next_page,
                        "skip_zero_qty_line_items": skip_zero_qty_line_items,
                        "sort": sort,
                        "status": status,
                        "window_size": window_size,
                    },
                    invoice_list_breakdowns_params.InvoiceListBreakdownsParams,
                ),
            ),
            model=InvoiceListBreakdownsResponse,
        )


class AsyncInvoicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncInvoicesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        skip_zero_qty_line_items: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvoiceRetrieveResponse:
        """
        Fetch a specific invoice for a given customer.

        Args:
          skip_zero_qty_line_items: If set, all zero quantity line items will be filtered out of the response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not invoice_id:
            raise ValueError(f"Expected a non-empty value for `invoice_id` but received {invoice_id!r}")
        return await self._get(
            f"/v1/customers/{customer_id}/invoices/{invoice_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"skip_zero_qty_line_items": skip_zero_qty_line_items},
                    invoice_retrieve_params.InvoiceRetrieveParams,
                ),
            ),
            cast_to=InvoiceRetrieveResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        credit_type_id: str | NotGiven = NOT_GIVEN,
        ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        skip_zero_qty_line_items: bool | NotGiven = NOT_GIVEN,
        sort: Literal["date_asc", "date_desc"] | NotGiven = NOT_GIVEN,
        starting_on: Union[str, datetime] | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Invoice, AsyncCursorPage[Invoice]]:
        """
        List all invoices for a given customer, optionally filtered by status, date
        range, and/or credit type.

        Args:
          credit_type_id: Only return invoices for the specified credit type

          ending_before: RFC 3339 timestamp (exclusive). Invoices will only be returned for billing
              periods that end before this time.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          skip_zero_qty_line_items: If set, all zero quantity line items will be filtered out of the response

          sort: Invoice sort order by issued_at, e.g. date_asc or date_desc. Defaults to
              date_asc.

          starting_on: RFC 3339 timestamp (inclusive). Invoices will only be returned for billing
              periods that start at or after this time.

          status: Invoice status, e.g. DRAFT, FINALIZED, or VOID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/invoices",
            page=AsyncCursorPage[Invoice],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "credit_type_id": credit_type_id,
                        "ending_before": ending_before,
                        "limit": limit,
                        "next_page": next_page,
                        "skip_zero_qty_line_items": skip_zero_qty_line_items,
                        "sort": sort,
                        "starting_on": starting_on,
                        "status": status,
                    },
                    invoice_list_params.InvoiceListParams,
                ),
            ),
            model=Invoice,
        )

    async def add_charge(
        self,
        *,
        customer_id: str,
        charge_id: str,
        customer_plan_id: str,
        description: str,
        invoice_start_timestamp: Union[str, datetime],
        price: float,
        quantity: float,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvoiceAddChargeResponse:
        """
        Add a one time charge to the specified invoice

        Args:
          charge_id: The Metronome ID of the charge to add to the invoice. Note that the charge must
              be on a product that is not on the current plan, and the product must have only
              fixed charges.

          customer_plan_id: The Metronome ID of the customer plan to add the charge to.

          invoice_start_timestamp: The start_timestamp of the invoice to add the charge to.

          price: The price of the charge. This price will match the currency on the invoice, e.g.
              USD cents.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return await self._post(
            f"/v1/customers/{customer_id}/addCharge",
            body=await async_maybe_transform(
                {
                    "charge_id": charge_id,
                    "customer_plan_id": customer_plan_id,
                    "description": description,
                    "invoice_start_timestamp": invoice_start_timestamp,
                    "price": price,
                    "quantity": quantity,
                },
                invoice_add_charge_params.InvoiceAddChargeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceAddChargeResponse,
        )

    def list_breakdowns(
        self,
        *,
        customer_id: str,
        ending_before: Union[str, datetime],
        starting_on: Union[str, datetime],
        credit_type_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        skip_zero_qty_line_items: bool | NotGiven = NOT_GIVEN,
        sort: Literal["date_asc", "date_desc"] | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        window_size: Literal["HOUR", "DAY"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[InvoiceListBreakdownsResponse, AsyncCursorPage[InvoiceListBreakdownsResponse]]:
        """
        List daily or hourly invoice breakdowns for a given customer, optionally
        filtered by status, date range, and/or credit type. Important considerations:

        - If we receive backdated usage after an invoice has been finalized, the
          backdated usage will be included in the response and usage numbers may differ.

        Args:
          ending_before: RFC 3339 timestamp. Breakdowns will only be returned for time windows that end
              on or before this time.

          starting_on: RFC 3339 timestamp. Breakdowns will only be returned for time windows that start
              on or after this time.

          credit_type_id: Only return invoices for the specified credit type

          limit: Max number of results that should be returned. For daily breakdowns, the
              response can return up to 35 days worth of breakdowns. For hourly breakdowns,
              the response can return up to 24 hours. If there are more results, a cursor to
              the next page is returned.

          next_page: Cursor that indicates where the next page of results should start.

          skip_zero_qty_line_items: If set, all zero quantity line items will be filtered out of the response

          sort: Invoice sort order by issued_at, e.g. date_asc or date_desc. Defaults to
              date_asc.

          status: Invoice status, e.g. DRAFT or FINALIZED

          window_size: The granularity of the breakdowns to return. Defaults to day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/invoices/breakdowns",
            page=AsyncCursorPage[InvoiceListBreakdownsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "starting_on": starting_on,
                        "credit_type_id": credit_type_id,
                        "limit": limit,
                        "next_page": next_page,
                        "skip_zero_qty_line_items": skip_zero_qty_line_items,
                        "sort": sort,
                        "status": status,
                        "window_size": window_size,
                    },
                    invoice_list_breakdowns_params.InvoiceListBreakdownsParams,
                ),
            ),
            model=InvoiceListBreakdownsResponse,
        )


class InvoicesResourceWithRawResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = to_raw_response_wrapper(
            invoices.retrieve,
        )
        self.list = to_raw_response_wrapper(
            invoices.list,
        )
        self.add_charge = to_raw_response_wrapper(
            invoices.add_charge,
        )
        self.list_breakdowns = to_raw_response_wrapper(
            invoices.list_breakdowns,
        )


class AsyncInvoicesResourceWithRawResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = async_to_raw_response_wrapper(
            invoices.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            invoices.list,
        )
        self.add_charge = async_to_raw_response_wrapper(
            invoices.add_charge,
        )
        self.list_breakdowns = async_to_raw_response_wrapper(
            invoices.list_breakdowns,
        )


class InvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = to_streamed_response_wrapper(
            invoices.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            invoices.list,
        )
        self.add_charge = to_streamed_response_wrapper(
            invoices.add_charge,
        )
        self.list_breakdowns = to_streamed_response_wrapper(
            invoices.list_breakdowns,
        )


class AsyncInvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = async_to_streamed_response_wrapper(
            invoices.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            invoices.list,
        )
        self.add_charge = async_to_streamed_response_wrapper(
            invoices.add_charge,
        )
        self.list_breakdowns = async_to_streamed_response_wrapper(
            invoices.list_breakdowns,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CustomerDetail", "CustomerConfig", "CurrentBillableStatus"]


class CustomerConfig(BaseModel):
    salesforce_account_id: Optional[str] = None
    """The Salesforce account ID for the customer"""


class CurrentBillableStatus(BaseModel):
    value: Literal["billable", "unbillable"]

    effective_at: Optional[datetime] = None


class CustomerDetail(BaseModel):
    id: str
    """the Metronome ID of the customer"""

    created_at: datetime
    """RFC 3339 timestamp indicating when the customer was created."""

    custom_fields: Dict[str, str]

    customer_config: CustomerConfig

    external_id: str
    """
    (deprecated, use ingest_aliases instead) the first ID (Metronome or ingest
    alias) that can be used in usage events
    """

    ingest_aliases: List[str]
    """
    aliases for this customer that can be used instead of the Metronome customer ID
    in usage events
    """

    name: str

    archived_at: Optional[datetime] = None
    """RFC 3339 timestamp indicating when the customer was archived.

    Null if the customer is active.
    """

    current_billable_status: Optional[CurrentBillableStatus] = None
    """This field's availability is dependent on your client's configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AlertRetrieveParams"]


class AlertRetrieveParams(TypedDict, total=False):
    alert_id: Required[str]
    """The Metronome ID of the alert"""

    customer_id: Required[str]
    """The Metronome ID of the customer"""

    plans_or_contracts: Literal["PLANS", "CONTRACTS"]
    """
    When parallel alerts are enabled during migration, this flag denotes whether to
    fetch alerts for plans or contracts.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CreditGrantEditParams"]


class CreditGrantEditParams(TypedDict, total=False):
    id: Required[str]
    """the ID of the credit grant"""

    credit_grant_type: str
    """the updated credit grant type"""

    expires_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """the updated expiration date for the credit grant"""

    name: str
    """the updated name for the credit grant"""
